/*------------------------------------------------------------------------
 Module:        hyperlink.c
 Author:        Alexander Stoica (Berlin, Germany)
 Project:       Hyperlink Control
 State:         Finished
 Creation Date: Aug 2002
 Description:   This is my first control i have written in C
                so there might be some errors. Send me a mail
                to nitex@freenet.de if you find some. For
                instructions on how to use this control see
                manual.html.

                Special thanks to John Findlay, who helped me
                a lot with his tips.
------------------------------------------------------------------------*/

#ifdef __LCC__
#pragma lib <winmm.lib>
#pragma lib <shell32.lib>
#pragma lib <shlwapi.lib>
#pragma lib <comctl32.lib>
#endif

#include <mmsystem.h>
#include <shellapi.h>
#include <shlwapi.h>
#include <commctrl.h>
#include "hyperlink.h"

LRESULT CALLBACK HypCtrlWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
static BOOL CreateTooltip(HWND hWnd);
static BOOL PlayClick(void);

// offsets for the window extra space
#define WND_HYPSTRUCT 0
#define WND_ISHOVER 4
#define WND_LEFT 6
#define WND_TOP 10
#define WND_RIGHT 14
#define WND_BOTTOM 18
#define WND_FONTN 22
#define WND_FONTH 26

static HINSTANCE hInst = NULL;
static UCHAR szHyperlinkClass[] = "clsHyperlink";


/*------------------------------------------------------------------------
 Procedure:     InitHypCtrl ID:1
 Purpose:       registers the hyperlink control
 Input:         if lpCursorName = NULL then the function will try to
                load a hand cursor from winhlp32.exe. if this fails a
                normal arrow will be used. if lpCursorName is given,
                then this cursor will be used
 Output:        returns TRUE if the hyperlink control was registered
                successfully
 Errors:        returns FALSE if the control could not be registered
------------------------------------------------------------------------*/
BOOL InitHypCtrl(HCURSOR lpCursorName = NULL)
{
	WNDCLASS wc;
	HCURSOR hCursor;

	hInst = GetModuleHandle(NULL);

	if (lpCursorName != NULL) {
		// a cursor was given so use it
		hCursor = lpCursorName;
	} else {
		UCHAR ucWinDir[MAX_PATH];
		HMODULE hModule;

		GetWindowsDirectory(ucWinDir, MAX_PATH);
		PathAppend(ucWinDir, "\\winhlp32.exe");

        if (hModule = LoadLibrary(ucWinDir)) {
			// try to load the hand cursor from winhlp32.exe
            HCURSOR hCursorTemp = LoadCursor(hModule, MAKEINTRESOURCE(106));
            if (hCursorTemp) hCursor = CopyCursor(hCursorTemp);
			FreeLibrary(hModule);
        } else {
			// the hand cursor could not be loaded so use the arrow instead
			hCursor = LoadCursor(hInst, IDC_ARROW);
		}
	}

	wc.style			= CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc		= (WNDPROC) HypCtrlWndProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 32;
	wc.hInstance		= hInst;
	wc.hbrBackground	= (HBRUSH) (HOLLOW_BRUSH);
	wc.lpszMenuName		= NULL;
	wc.hIcon			= NULL;
	wc.hCursor			= hCursor;
	wc.lpszClassName	= szHyperlinkClass;

	if (!RegisterClass(&wc)) return FALSE;

	// initialize common controls for the tooltips
	InitCommonControls();

	return TRUE;
}

/*------------------------------------------------------------------------
 Procedure:     InitHypCtrlStruct ID:1
 Purpose:       initializes the hyperlink structure with default
                values
------------------------------------------------------------------------*/
void InitHypCtrlStruct(HYPCTRL* pHc)
{
	pHc->ucLink			= NULL;				// link
	pHc->ucText			= NULL;				// text displayed as link
	pHc->ucTooltip		= NULL;				// tooltip text
	pHc->fgcNormal		= RGB(0, 0, 238);	// blue
	pHc->fgcHover		= RGB(255, 0, 0);	// red
	pHc->bgcNormal		= 0x01000000;		// no color
	pHc->bgcHover		= 0x01000000;		// no color
	pHc->ulStyle		= ulAlways;			// underline always
	pHc->dtStyle		= DT_LEFT | DT_TOP; // text alignment
	pHc->bAutoSize		= TRUE;				// control resizes to fit the text
	pHc->bPlayClick		= TRUE;				// play sound on mouseclick
}

/*------------------------------------------------------------------------
 Procedure:     CreateHypCtrl ID:1
 Purpose:       creates a hyperlink control
 Input:         see manual.html
 Output:        returns handle to hyperlink control
 Errors:        returns NULL if the control could not be created
------------------------------------------------------------------------*/
HWND CreateHypCtrl(HWND hWndParent, HYPCTRL* pHc, int x, int y, int width, int height)
{
	HYPCTRL* pHcWnd;
	HWND hWndHyperlink;
	RECT rect;
	LOGFONT lf;
	HFONT hFontParent, hFontNormal, hFontHover;

	// some error checking
	if (hInst == NULL) return (HWND) NULL;
	if (hWndParent == NULL) return (HWND) NULL;
	if (pHc->ucLink == NULL) return (HWND) NULL;

	hWndHyperlink = CreateWindow(szHyperlinkClass, szHyperlinkClass, WS_CHILD,
		x, y, width, height, hWndParent, NULL, hInst, NULL);

	if (!hWndHyperlink) return (HWND) NULL;

	// alloc some space to hold all hyperlink stuff
	if ((pHcWnd = malloc(sizeof(HYPCTRL))) == NULL) {
		DestroyWindow(hWndHyperlink);
		return (HWND) NULL;
	}

	CopyMemory(pHcWnd, pHc, sizeof(HYPCTRL));

	// copy the text entries from HYPCTRL
	if ((pHcWnd->ucLink = malloc(lstrlen(pHc->ucLink)+1)) == NULL) {
		if (pHcWnd) free(pHcWnd);
		DestroyWindow(hWndHyperlink);
		return (HWND) NULL;
	}

	lstrcpy(pHcWnd->ucLink, pHc->ucLink);

	if (pHcWnd->ucText != NULL) {
		if ((pHcWnd->ucText = malloc(lstrlen(pHc->ucText)+1)) == NULL) {
			if (pHcWnd->ucLink) free(pHcWnd->ucLink);
			if (pHcWnd) free(pHcWnd);
			DestroyWindow(hWndHyperlink);
			return (HWND) NULL;
		}
		lstrcpy(pHcWnd->ucText, pHc->ucText);
	} else {
		pHcWnd->ucText = pHcWnd->ucLink;
	}

	if (pHcWnd->ucTooltip != NULL) {
		if ((pHcWnd->ucTooltip = malloc(lstrlen(pHc->ucTooltip)+1)) == NULL) {
			if (pHcWnd->ucLink) free(pHcWnd->ucLink);
			if (pHcWnd->ucText) free(pHcWnd->ucText);
			if (pHcWnd) free(pHcWnd);
			DestroyWindow(hWndHyperlink);
			return (HWND) NULL;
		}
		lstrcpy(pHcWnd->ucTooltip, pHc->ucTooltip);
	}

	// save a pointer to the structure in the window extra space
	SetWindowLong(hWndHyperlink, WND_HYPSTRUCT, (LONG) (pHcWnd));
	SetWindowWord(hWndHyperlink, WND_ISHOVER, (WORD) FALSE);

	// create the fonts for the control
	hFontParent = (HFONT) SendMessage(GetParent(hWndHyperlink), WM_GETFONT, 0, 0);

	if (hFontParent != NULL) {
		GetObject(hFontParent, sizeof(LOGFONT), &lf);

		switch (pHcWnd->ulStyle) {
			case ulHover:
				hFontNormal = CreateFontIndirect(&lf);
				lf.lfUnderline = (BYTE) TRUE;
				hFontHover  = CreateFontIndirect(&lf);
				break;
			case ulAlways:
				lf.lfUnderline = (BYTE) TRUE;
				hFontNormal = CreateFontIndirect(&lf);
				hFontHover  = CreateFontIndirect(&lf);
				break;
			case ulNone:
				hFontNormal = CreateFontIndirect(&lf);
				hFontHover  = CreateFontIndirect(&lf);
				break;
			}
			// save the fonts in the window extra space
			SetWindowLong(hWndHyperlink, WND_FONTN, (LONG) hFontNormal);
			SetWindowLong(hWndHyperlink, WND_FONTH, (LONG) hFontHover);
	} else {
		// use the system font
		SetWindowLong(hWndHyperlink, WND_FONTN, (LONG) NULL);
		SetWindowLong(hWndHyperlink, WND_FONTH, (LONG) NULL);
	}

	GetClientRect(hWndHyperlink, &rect);

	// adjust window size to fit the text
	if (pHcWnd->bAutoSize) {
		PAINTSTRUCT ps;
		HDC hdc;
		SIZE size;

		hdc	= BeginPaint(hWndHyperlink, &ps);
		SelectObject(hdc, (HFONT) GetWindowLong(hWndHyperlink, WND_FONTN));
		GetTextExtentPoint32(hdc, pHcWnd->ucText, lstrlen(pHcWnd->ucText), &size);
		rect.right	= size.cx - rect.left;
		rect.bottom	= size.cy - rect.top;
		EndPaint(hWndHyperlink, &ps);

		SetWindowPos(hWndHyperlink, 0, 0, 0, size.cx, size.cy, SWP_NOMOVE | SWP_NOZORDER);
	}

	// save window size in the window extra space
	SetWindowLong(hWndHyperlink, WND_LEFT, (LONG) (rect.left));
	SetWindowLong(hWndHyperlink, WND_TOP, (LONG) (rect.top));
	SetWindowLong(hWndHyperlink, WND_RIGHT, (LONG) (rect.right));
	SetWindowLong(hWndHyperlink, WND_BOTTOM, (LONG) (rect.bottom));

	// create tooltip if requested
	if (pHcWnd->ucTooltip != NULL) CreateTooltip(hWndHyperlink);

	// show the window
	ShowWindow(hWndHyperlink, SW_NORMAL);
	UpdateWindow(hWndHyperlink);

	return hWndHyperlink;
}

/*------------------------------------------------------------------------
 Procedure:     HypCtrlWndProc ID:1
 Purpose:       handles all hyperlink control messages
------------------------------------------------------------------------*/
LRESULT CALLBACK HypCtrlWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg){
		case WM_LBUTTONUP: {
			HYPCTRL* pHcWnd = pHcWnd = (HYPCTRL*) GetWindowLong(hWnd, WND_HYPSTRUCT);
			// play click sound
			if (pHcWnd->bPlayClick) PlayClick();
			SetWindowWord(hWnd, WND_ISHOVER, (WORD) FALSE);
			// jump to link
			ShellExecute(NULL, "open", pHcWnd->ucLink, NULL, NULL, SW_SHOW);
			// force redrawing so that the control looses the hover state
			InvalidateRect(hWnd, NULL, TRUE);
			return FALSE;
		}
		case WM_PAINT: {
			PAINTSTRUCT ps;
			HDC hdc;
			RECT rect;

			HYPCTRL* pHcWnd = (HYPCTRL*) GetWindowLong(hWnd, WND_HYPSTRUCT);
			BOOL bIsOverControl = (BOOL) GetWindowWord(hWnd, WND_ISHOVER);

			rect.left	= (LONG) GetWindowLong(hWnd, WND_LEFT);
			rect.top	= (LONG) GetWindowLong(hWnd, WND_TOP);
			rect.right	= (LONG) GetWindowLong(hWnd, WND_RIGHT);
			rect.bottom	= (LONG) GetWindowLong(hWnd, WND_BOTTOM);

			hdc	= BeginPaint(hWnd, &ps);

			if (bIsOverControl) {
				// draws hover state
				if (pHcWnd->bgcHover != 0x01000000) {
					LOGBRUSH lb;
					lb.lbStyle = BS_SOLID;
					lb.lbColor = pHcWnd->bgcHover;
					HBRUSH hBrush = CreateBrushIndirect(&lb);
				    FillRect(hdc, &rect, hBrush);
					DeleteObject(hBrush);
				}
				SelectObject(hdc, (HFONT) GetWindowLong(hWnd, WND_FONTH));
				SetTextColor(hdc, pHcWnd->fgcHover);
			} else {
				// draws normal state
				if (pHcWnd->bgcNormal != 0x01000000) {
					LOGBRUSH lb;
					lb.lbStyle = BS_SOLID;
					lb.lbColor = pHcWnd->bgcNormal;
					HBRUSH hBrush = CreateBrushIndirect(&lb);
				    FillRect(hdc, &rect, hBrush);
					DeleteObject(hBrush);
				}
				SelectObject(hdc, (HFONT) GetWindowLong(hWnd, WND_FONTN));
				SetTextColor(hdc, pHcWnd->fgcNormal);
			}
			SetBkMode(hdc, TRANSPARENT);
			// draw the text
			DrawText(hdc, pHcWnd->ucText, lstrlen(pHcWnd->ucText), &rect, pHcWnd->dtStyle);

			EndPaint(hWnd, &ps);
			return FALSE;
		}
		case WM_MOUSEMOVE: {
			RECT rect;

			rect.left	= (LONG) GetWindowLong(hWnd, WND_LEFT);
			rect.top	= (LONG) GetWindowLong(hWnd, WND_TOP);
			rect.right	= (LONG) GetWindowLong(hWnd, WND_RIGHT);
			rect.bottom	= (LONG) GetWindowLong(hWnd, WND_BOTTOM);

			// check if mouse is over control
			if ((BOOL) GetWindowWord(hWnd, WND_ISHOVER)) {
				POINT ptMousePos;
				DWORD dwMousePos;

				dwMousePos = GetMessagePos();

				ptMousePos.x = LOWORD(dwMousePos);
				ptMousePos.y = HIWORD(dwMousePos);

				ScreenToClient(hWnd, &ptMousePos);

				if (!(BOOL) PtInRect(&rect, ptMousePos)) {
					SetWindowWord(hWnd, WND_ISHOVER, (WORD) FALSE);
				    ReleaseCapture();
					InvalidateRect(hWnd, &rect, TRUE);
				}
			} else {
				SetWindowWord(hWnd, WND_ISHOVER, (WORD) TRUE);
				SetCapture(hWnd);
				InvalidateRect(hWnd, &rect, TRUE);
			}
			return FALSE;
		}
		case WM_CLOSE: {
			HYPCTRL* pHcWnd = (HYPCTRL*) GetWindowLong(hWnd, WND_HYPSTRUCT);
			HFONT hFontTemp;

			// clean up
			if (pHcWnd->ucLink) free(pHcWnd->ucLink);
			if (pHcWnd->ucText) free(pHcWnd->ucText);
			if (pHcWnd->ucTooltip) free(pHcWnd->ucTooltip);

			hFontTemp = (HFONT) GetWindowLong(hWnd, WND_FONTN);
			if (hFontTemp) DeleteObject(hFontTemp);

			hFontTemp = (HFONT) GetWindowLong(hWnd, WND_FONTH);
			if (hFontTemp) DeleteObject(hFontTemp);

			if (pHcWnd) free(pHcWnd);
			DestroyWindow(hWnd);
			return FALSE;
		}
	}
	return DefWindowProc(hWnd, msg, wParam, lParam);
}

/*------------------------------------------------------------------------
 Procedure:     CreateTooltip ID:1
 Purpose:       creates a tooltip for a hyperlink control
 Input:         handle of hperlink window
 Output:        returns TRUE if the tooltip was created successfully
 Errors:        returns FALSE if the tooltip could not be created
------------------------------------------------------------------------*/
static BOOL CreateTooltip(HWND hWnd)
{
    HWND hWndTT;
    TOOLINFO ti;

    hWndTT = CreateWindowEx(WS_EX_TOPMOST, TOOLTIPS_CLASS, NULL, WS_POPUP | TTS_NOPREFIX | TTS_ALWAYSTIP,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, hWnd, NULL, hInst, NULL);

	if (!hWndTT) return FALSE;

    SetWindowPos(hWndTT, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);

	HYPCTRL* pHcWnd = (HYPCTRL*) GetWindowLong(hWnd, WND_HYPSTRUCT);

    ti.cbSize = sizeof(TOOLINFO);
    ti.uFlags = TTF_SUBCLASS;
    ti.hwnd = hWnd;
    ti.hinst = hInst;
    ti.uId = 0;
	// get text for tooltip
    ti.lpszText = pHcWnd->ucTooltip;
    ti.rect.left =  (LONG) GetWindowLong(hWnd,  WND_LEFT);
    ti.rect.top = (LONG) GetWindowLong(hWnd, WND_TOP);
    ti.rect.right = (LONG) GetWindowLong(hWnd, WND_RIGHT);
    ti.rect.bottom = (LONG) GetWindowLong(hWnd, WND_BOTTOM);

	// add tooltip
    SendMessage(hWndTT, TTM_ADDTOOL, 0, (LPARAM) (LPTOOLINFO) &ti);
	return TRUE;
}

/*------------------------------------------------------------------------
 Procedure:     PlayClick ID:1
 Purpose:       plays a click sound when clicked on a hyperlink
                control
------------------------------------------------------------------------*/
static BOOL PlayClick(void)
{
	// click.wav: 22050hz, 16bit, mono
	static const UCHAR wavClick[] = {
		0x52,0x49,0x46,0x46,0x72,0x05,0000,0000,0x57,0x41,0x56,0x45,0x66,0x6d,0x74,0x20,
		0x10,0000,0000,0000,0x01,0000,0x01,0000,0x22,0x56,0000,0000,0x44,0xac,0000,0000,
		0x02,0000,0x10,0000,0x64,0x61,0x74,0x61,0x4e,0x05,0000,0000,0xb3,0x01,0xf8,0000,
		0x32,0000,0xde,0x01,0x22,0x01,0000,0xfe,0x3e,0xff,0xe4,0xfe,0x48,0xfe,0x80,0x03,
		0x04,0x07,0x68,0xf7,0x2f,0x0e,0x88,0x06,0x77,0xe2,0x2c,0x1f,0x71,0xfa,0xaf,0xc9,
		0x6f,0x04,0x58,0x23,0xc8,0x36,0x2f,0x12,0xb9,0xc5,0x32,0xd8,0xc2,0x1c,0x7b,0x39,
		0xdd,0xf3,0xd6,0xf2,0x9f,0xe2,0x53,0x03,0xcc,0x45,0x8c,0xe5,0x8c,0xcb,0x68,0x14,
		0x74,0x28,0xb5,0x01,0x7f,0xd9,0xf4,0xe7,0xee,0x0f,0x8a,0x29,0x12,0x06,0xb6,0xdb,
		0x54,0x01,0x3c,0x0b,0000,0000,0x8d,0x0e,0x22,0x08,0x6f,0x04,0x29,0xed,0x70,0xe8,
		0000,0000,0xfb,0xf8,0000,0000,0000,0000,0x2c,0x0a,0xc3,0x05,0x5d,0xfb,0xaf,0x03,
		0xbd,0x03,0xd8,0x0e,0x88,0x15,0x5a,0xff,0xb1,0xf5,0x39,0xf6,0x29,0xf8,0000,0000,
		0000,0000,0xc5,0xfd,0x9d,0xfe,0x8f,0x04,0x63,0xfe,0x79,0x07,0x7b,0x0d,0x6d,0xfd,
		0x48,0xfb,0x4b,0x06,0xd4,0x03,0x61,0xf6,0xf6,0xfd,0x46,0xfd,0x07,0x02,0x2a,0xfa,
		0xda,0xee,0xd3,0x0a,0x3c,0x0e,0x3e,0x04,0xa4,0x07,0x06,0000,0xe9,0xfc,0000,0000,
		0xf4,0x04,0xcd,0xfd,0xff,0xf1,0x99,0xfd,0x0c,0x08,0x2a,0x05,0xd3,0xef,0x60,0xfb,
		0x67,0x11,0xe9,0x0b,0x55,0xfd,0x99,0xfd,0x4e,0x04,0x01,0000,0x88,0xfc,0x8e,0x01,
		0x6c,0xfa,0x0c,0xf4,0x80,0xff,0xa3,0x07,0x31,0xff,0xad,0xf7,0x7c,0x07,0x4a,0x0d,
		0000,0000,0xfa,0xfa,0x72,0xff,0x8b,0x04,0000,0000,0000,0000,0x14,0xfd,0x7a,0xf7,
		0x74,0xf8,0x6b,0x04,0x54,0x09,0xfb,0xfd,0xe2,0xfd,0x80,0x06,0xa6,0x0c,0x55,0x02,
		0xc7,0xfa,0x8c,0xfb,0xd7,0000,0xd4,0x02,0xd5,0xff,0x55,0xfe,0x9c,0xf6,0xde,0xfa,
		0x66,0x04,0x8d,0x08,0x38,0xff,0xa2,0xfc,0x3c,0x07,0xaa,0x02,0000,0000,0x15,0x03,
		0x1e,0000,0x68,0xf8,0x41,0xfa,0x24,0xfe,0xba,0x02,0xe3,0xfd,0x9b,0x01,0xce,0x04,
		0x8e,0xfd,0x20,0xfd,0xe8,0x05,0xc0,0x07,0x50,0xfe,0x84,0xf9,0xf0,0xff,0xe3,0x03,
		0x1a,0000,0xd5,0xfc,0x1e,0xfe,0x23,0xff,0x3a,0x02,0xb9,0x06,0xc2,0x02,0xbf,0xf8,
		0xe2,0xff,0x2d,0x08,0x4c,0x04,0x4f,0xfc,0x70,0xfb,0000,0000,0000,0000,0x34,0xff,
		0x41,0xff,0x4f,0x01,0x2c,0x01,0x99,0x02,0x4e,0x01,0x6d,0xff,0x02,0x01,0x76,0000,
		0x6c,0xfe,0x92,0xfd,0xfb,0xff,0xca,0000,0x4d,0xfe,0x0c,0xfd,0x56,0000,0xd6,0x02,
		0x86,0x02,0xfc,0xff,0x60,0xfe,0xda,0x01,0xc2,0x05,0x4c,0x02,0x7d,0xfc,0x3a,0xfb,
		0x88,0000,0xc4,0x01,0xc8,0xfe,0x08,0xfe,0x51,0x01,0x75,0x03,0x89,0x01,0x12,0xff,
		0x2e,0xff,0x39,0x02,0x69,0x03,0x92,0x01,0x39,0xfc,0xa8,0xfc,0x04,0x01,0x20,0x01,
		0x5e,0xfe,0x32,0xfe,0xa8,0000,0x7b,0x02,0xd2,0x01,0x1d,0000,0x10,0xff,0x60,0000,
		0x2e,0x03,0xdb,0x01,0x47,0xff,0xe1,0xfc,0xa3,0xfd,0x64,0x01,0xf9,0x01,0x0d,0000,
		0xf8,0xfd,0x54,0000,0xdc,0x03,0x78,0x03,0xd3,0000,0x2c,0xff,0000,0000,0000,0000,
		0xa2,0xff,0x18,0xfe,0x86,0xfd,0x0c,0xff,0x41,0x01,0x23,0x01,0x53,0xff,0x4d,0000,
		0x29,0x02,0xfc,0x02,0x94,0000,0x7c,0xff,0000,0000,0x6a,0000,0x42,0xff,0x57,0xfe,
		0xa0,0xfd,0xac,0xfe,0xe8,0x01,0xc2,0000,0xea,0xff,0xb2,0xff,0x50,0x02,0xbc,0x03,
		0xbc,0x01,0x5c,0xfe,0x9e,0xfe,0000,0000,0x9c,0xff,0x47,0xfe,0xfc,0xfd,0xeb,0xfe,
		0x54,0x01,0xfc,0x02,0xe6,0000,0000,0000,0x23,0x01,0x52,0x03,0xea,0x01,0x64,0xff,
		0x74,0xff,0000,0000,0x70,0000,0x38,0xff,0x43,0xfe,0x62,0xff,0x18,0x01,0x36,0x01,
		0x81,0x01,0xc4,0000,0x74,0000,0x08,0x02,0x42,0x01,0x22,0xff,0xd9,0xfe,0000,0000,
		0x4f,0000,0x74,0xff,0xca,0xfe,0x37,0000,0xb9,0x01,0x2e,0x02,0x60,0x01,0000,0000,
		0x46,0000,0xf0,0x01,0x4a,0x01,0x91,0xff,0x3a,0xff,0x50,0xff,0000,0000,0000,0000,
		0xc2,0xff,0x5b,0xff,0x93,0000,0xfe,0x01,0x2c,0x02,0xa7,0000,0xb6,0xff,0x41,0000,
		0xaf,0000,0x02,0000,0x45,0xff,0x85,0xff,0x4a,0xff,0x1f,0000,0x55,0000,0x7f,0000,
		0xed,0xff,0x66,0000,0x5b,0x01,0x13,0x01,0x62,0000,0x58,0000,0xc3,0000,0xdd,0xff,
		0x08,0xff,0x20,0xff,0xa5,0000,0x8f,0000,0000,0000,0xd9,0xff,0x82,0000,0x6d,0x01,
		0x11,0x01,0x7c,0000,0000,0xff,0x06,0000,0x17,0x01,0x5c,0000,0000,0xff,0x04,0xff,
		0000,0000,0x4d,0000,0x45,0000,0x3d,0000,0x6c,0000,0xd0,0xff,0x2b,0000,0x56,0x01,
		0xf0,0000,0x2c,0xff,0x15,0xff,0x70,0000,0xea,0000,0x68,0000,0000,0000,0000,0000,
		0000,0000,0xbd,0000,0x06,0x01,0x42,0000,0x92,0xff,0x52,0000,0xbc,0000,0x23,0000,
		0x8f,0000,0x47,0000,0xc7,0xff,0xf2,0xff,0x0c,0x01,0xbd,0000,0000,0000,0000,0000,
		0x05,0x01,0x24,0x01,0000,0000,0xcb,0xff,0x2f,0000,0x64,0000,0x25,0000,0x6f,0000,
		0x63,0000,0xd1,0xff,0xb7,0xff,0x8c,0000,0x93,0000,0000,0000,0000,0000,0x84,0000,
		0x01,0x01,0x75,0000,0x43,0000,0x05,0000,0x18,0000,0x1d,0000,0x7e,0000,0x85,0000,
		0000,0000,0x22,0000,0x7d,0000,0xfe,0000,0x98,0000,0x24,0000,0x7a,0000,0x6a,0000,
		0x17,0000,0x2c,0000,0x46,0000,0x31,0000,0x30,0000,0x70,0000,0x30,0000,0xe7,0xff,
		0x16,0000,0x16,0000,0xb8,0000,0x70,0000,0x26,0000,0000,0000,0x9b,0000,0x9c,0000,
		0x2c,0000,0x2e,0000,0000,0000,0x81,0000,0xcc,0000,0xb4,0000,0x2f,0000,0xed,0xff,
		0xad,0000,0xab,0000,0xfa,0xff,0xdd,0xff,0x6c,0000,0x62,0x01,0x62,0000,0x67,0xff,
		0x99,0000,0xf2,0000,0x05,0000,0xd8,0xff,0000,0000,0000,0000,0x7b,0000,0x60,0000,
		0000,0000,0xd4,0xff,0x26,0000,0x48,0000,0xd1,0xff,0xf0,0xff,0x22,0000,0x60,0000,
		0xd0,0xff,0x24,0000,0xc3,0000,0x5a,0000,0000,0000,0x8a,0000,0xbc,0000,0xca,0000,
		0x12,0000,0000,0000,0000,0000,0000,0000,0x22,0000,0x19,0000,0x90,0xff,0x7b,0xff,
		0xcc,0000,0xb6,0000,0000,0000,0x37,0000,0000,0000,0x2c,0000,0000,0000,0x66,0000,
		0x9a,0000,0xf1,0xff,0x9a,0xff,0x47,0000,0xa6,0000,0x08,0000,0xca,0xff,0x57,0000,
		0x87,0000,0x6f,0000,0x6e,0000,0x12,0000,0xaa,0xff,0xb4,0xff,0x55,0000,0x55,0000,
		0x62,0xff,0xbc,0xff,0x22,0000,0x4e,0000,0x68,0000,0x13,0x01,0x02,0x01,0x32,0000,
		0xaf,0xff,0xaf,0000,0x34,0x01,0x42,0000,0xa8,0xff,0x19,0000,0x81,0000,0x2a,0000,
		0x87,0000,0xda,0000,0xdb,0xff,0x42,0000,0x63,0x01,0xc4,0000,0x90,0xff,0x64,0xff,
		0x36,0x01,0x75,0000,0xe0,0xfe,0x8c,0xff,0xc2,0xff,0x6e,0000,0x7b,0000,0x58,0000,
		0x80,0000,0000,0000,0x7c,0000,0x56,0x01,0xf9,0000,0x37,0000,0x76,0000,0xb9,0000,
		0x27,0000,0xd6,0xff,0xe1,0xff,0x57,0000,0x30,0000,0x39,0000,0x5a,0000,0x08,0000,
		0x33,0000,0x9c,0000,0xd1,0000,0x16,0000,0x98,0xff,0x9c,0000,0xa2,0000,0xa4,0xff,
		0xd3,0xfe,0x7f,0xff,0x86,0000,0x44,0000,0xd2,0xff,0x60,0xff,0xfc,0000,0x8e,0x01,
		0x09,0x01,0x38,0000,0000,0000,0000,0000,0000,0000,0x2b,0000,0x18,0000,0x86,0xff,
		0xd0,0xff,0x78,0000,0x5a,0000,0x14,0000,0xa9,0000,0xd4,0000,0xc1,0000,0xab,0000,
		0x84,0000,0x1f,0000,0xd6,0xff,0xc4,0xff,0x80,0xff,0x16,0xff,0x90,0xff,0x21,0000,
		0x3f,0000,0x52,0000,0x4b,0000,0x84,0000,0x33,0000,0x28,0000,0x45,0000,0x42,0000,
		0xe0,0xff,0xd0,0xff,0x10,0000,0xe0,0xff,0xe0,0xff,0x20,0000,0x50,0000,0x40,0000,
		0x63,0000,0xfc,0000,0xba,0000,0x54,0000,0x6d,0000,0x1f,0000,0000,0000,0x1b,0000,
		0x45,0000,0x21,0000,0xdc,0xff,0x40,0000,0xac,0000,0x40,0000,0xc7,0xff,0x3c,0000,
		0xbf,0000,0x76,0000,0x0a,0000,0x9d,0xff,0xd0,0xff,0x6f,0000,0xae,0000,0x51,0000,
		0000,0000,0000,0000,0x6a,0000,0xa5,0000,0x36,0000,0x9c,0xff,0x6e,0xff,0x02,0000,
		0x72,0000,0x26,0000,0x12,0000,0x70,0000,0xe5,0000,0xa1,0x01,0x07,0x01,0x39,0000,
		0000,0000,0x64,0000,0xd0,0000,0x1f,0000,0x59,0xff,0xaf,0xff,0x4e,0000,0x42,0000,
		0x39,0000,0x32,0000,0x50,0000,0x20,0000,0x30,0000,0x60,0000,0x76,0000,0x19,0000,
		0x20,0000,0x36,0000,0xe0,0xff,0x10,0000,0x80,0000,0x98,0000,0x91,0000,0x36,0000,
		0x17,0000,0x51,0000,0xd1,0000,0x2c,0x01,0xb0,0000};

// plays click sound
return PlaySound(wavClick, hInst, SND_ASYNC | SND_MEMORY | SND_NODEFAULT | SND_NOWAIT);
}


